# Generated by Django 2.1.1 on 2018-09-20 12:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('translations', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='translation',
            name='language',
            field=models.CharField(choices=[('en', 'English'), ('en-gb', 'English (Great Britain)'), ('de', 'German'), ('tr', 'Turkish')], help_text='the language of the translation', max_length=32, verbose_name='language'),
        ),
    ]
